# BSVAR-subset-flatprior.R
#
# Impulse responses for flat prior model
#
# Brandt and Freeman BVAR 2 paper -- APSA and PA
#
# Patrick Brandt
# pbrandt@utdallas.edu

# Load libraries
library(MSBVAR)  # Brandt BVAR / VAR library

# Load data and functions
load("Data_Setup.RData")

# Subsets we want -- separate out the political and econ shocks --
# note that these are stored in column / outer array order for the IRF
# for the full 9 x 9.

sublist1 <- c(55:60, 64:69, 73:78)  # Responses to political shocks

sublist2 <- c(7:9, 16:18, 25:27, 34:36, 43:45, 52:54, 61:63, 70:72,
              79:81) # Responses to economic shocks

# Load in the impulse responses for the various priors and subset
# results


# Now do all of the same for the flat priors.

load("flatpriorirfs.RData")
flatprior.responses2politics <- 100*flatprior.irfs[,,sublist1]
flatprior.responses2econ <- 100*flatprior.irfs[,,sublist2]
rm(flatprior.irfs)
gc()


# Flat prior responses to political shocks
pdf(file="flatresponses2econ.pdf", width=9, height=4)

plot.mc.irf.svar.subset(list(flatprior.responses2econ), num.eqn=3,
                        num.shk=9, method = c("Sims-Zha2"),
                        component = 1,
                        probs = c(0.16, 0.84),
                        eqnnames = c("CS", "Approval", "MP"),
                        shknames = c("Pcom", "M2", "R", "Y", "CPI",
                        "U", "CS", "A", "MP"))

dev.off()
gc()

# Flat prior responses to political shocks
pdf(file="flatresponses2politics.pdf", width=5, height=8)

plot.mc.irf.svar.subset(list(flatprior.responses2politics), num.eqn=6,
                        num.shk=3, method = c("Sims-Zha2"),
                        component = 1,
                        probs = c(0.16, 0.84),
                        eqnnames = c("Pcom", "M2", "R", "Y", "CPI",
                        "U"),
                        shknames = c("CS", "Approval", "MP"))

dev.off()
gc()

# Flat prior responses with 90% error bands
pdf(file="flatresponses2politics90.pdf", width=5, height=8)

plot.mc.irf.svar.subset(list(flatprior.responses2politics), num.eqn=6,
                        num.shk=3, method = c("Sims-Zha2"),
                        component = 1,
                        probs = c(0.05, 0.95),
                        eqnnames = c("Pcom", "M2", "R", "Y", "CPI",
                        "U"),
                        shknames = c("CS", "Approval", "MP"))

dev.off()
gc()

pdf(file="flatresponses2econ90.pdf", width=9, height=4)

plot.mc.irf.svar.subset(list(flatprior.responses2econ), num.eqn=3,
                        num.shk=9, method = c("Sims-Zha2"),
                        component = 1,
                        probs = c(0.05, 0.95),
                        eqnnames = c("CS", "Approval", "MP"),
                        shknames = c("Pcom", "M2", "R", "Y", "CPI",
                        "U", "CS", "A", "MP"))

dev.off()
gc()

save.image(file="flatpriorirfs-output.RData")

q(save="no")
