# modelcomparisons.R
#
# Compares results of the B-SVAR models
#
# Patrick T. Brandt
#
# 2006-04-18 : revised to use new version of MSBVAR
# 2006-08-22 : includes all models from sensitivity analysis
#
#
#

library(MSBVAR)

set.seed(789143)

##############################################
# Running tally model
##############################################

# Load posterior objects
load("runningtallyposterior.RData")

# Compute the posterior summaries

print(system.time(running.tally.fit <- posterior.fit(running.tally.model,
                                          running.tally.A0)))

rm(running.tally.A0)

##############################################
# Midrange model
##############################################

# Load posterior objects
load("midrangeposterior.RData")

# Compute the posterior summaries

print(system.time(midrange.fit <- posterior.fit(midrange.model,
                                                midrange.A0)))

rm(midrange.A0)

##############################################
# Best model
##############################################

# Load posterior objects
load("bestposterior.RData")

# Compute the posterior summaries

print(system.time(best.fit <- posterior.fit(best.model,
                                            best.A0)))

rm(best.A0)


##############################################
# Best ML model
##############################################

# Load posterior objects
load("bestMLposterior.RData")

# Compute the posterior summaries

print(system.time(bestML.fit <- posterior.fit(bestML.model,
                                            bestML.A0)))

rm(bestML.A0)


##############################################
# Flat prior model
##############################################

# Load posterior objects
load("flatpriorposterior.RData")

# Compute the posterior summaries

print(system.time(flatprior.fit <- posterior.fit(flatprior.model,
                                            flatprior.A0)))

rm(flatprior.A0)

# Save and exit

save.image(file="fits.RData", compress=T)
q(save="no")
