## "posterior.fit" <- function(varobj, A0.posterior.obj=NULL)
## {
##     if(inherits(varobj, "VAR"))
##     {

##       stop("posterior.fit() not implemented for VAR class objects.\n")
## ##         output <- posterior.fit.VAR(varobj)
## ##         attr(output, "class") <- c("posterior.fit.VAR")
## ##         return(output)
##     }
##     if(inherits(varobj, "BVAR"))
##     {
##         output <- posterior.fit.BVAR(varobj)
##         attr(output, "class") <- c("posterior.fit.BVAR")
##         return(output)
##     }
##     if(inherits(varobj, "BSVAR"))
##     {
##         output <- posterior.fit.BSVAR(varobj, A0.posterior.obj)
##         attr(output, "class") <- c("posterior.fit.BSVAR")
##         return(output)
##     }
## }


"A0.get" <- function(A0,index){
    if(index==1) {st <- 1} else {st <- 1 + (index-1)*length(A0$struct)}
    A0.out <- vector(mode="numeric",length=(A0$m*A0$m))
    A0.out[A0$struct] <- A0$A0[st:(st+length(A0$struct)-1)]
    return(matrix(A0.out,nrow=A0$m))
}

"W.get" <- function(W,index)
{
    start.index <- (index-1) * W$m + 1
    end.index <- start.index + W$m - 1
    Wout <- vector(mode="list",length=W$m)
    if(index==1){tmp.index <- 0}else{tmp.index <- W$W.index[(start.index-1)]}
    for(i in start.index:end.index){
        dim.w <- sqrt(W$W.index[i]-tmp.index)
        w <- matrix(W$W[(tmp.index+1):W$W.index[i]],nrow=dim.w)
        Wout[[(i-start.index+1)]] <- w
        tmp.index <- W$W.index[i]
    }
    return(Wout)
}


# TRANSLATION FUNCTIONS FOR SQUEEZED PARAMETERS IN structural B-SVAR
# models:

# Some utility functions for b2a and a2b.  These convert from the
# squeezed parameter vector b_i to a_i using b_i = U_i' a_i as n
# footnote 11 of W&Z 2003.

"b2a" <- function(b, Ui)
{ m <- nrow(as.matrix(Ui[[1]]))
  n0 <- sapply(1:m, function(i) {ncol(as.matrix(Ui[[i]]))})
  n0cum <- c(0,cumsum(n0))
  A0 <- matrix(0,m,m)

  for(i in 1:m)
    {
      A0[,i] <- as.matrix(Ui[[i]])%*%b[(n0cum[i]+1):(n0cum[(i+1)])]
    }

  return(A0)
}

# A0 = full A0 matrix....
"a2b" <- function(A0, Ui)
  { m <- nrow(A0)
    n0 <- sapply(1:m, function(i) {ncol(as.matrix(Ui[[i]]))})
    n0cum <- c(0,cumsum(n0))
    b <- matrix(0, n0cum[length(n0cum)], 1)

    for(i in 1:m)
      {
        b[(n0cum[i]+1):(n0cum[(i+1)])] <- t(as.matrix(Ui[[i]]))%*%A0[,i]
      }
    return(b)
  }



"posterior.pdf" <- function(varobj, A0.posterior.obj)
{
 # Constants from the model object

    m <- ncol(varobj$A0.mode)
    gk <- varobj$F.posterior
    N2 <- A0.posterior.obj$N2

   # compute the pdf of the prior: p(A_0, A_+) and the exponential
   # term of pr(Y|A0, A+)

    yexpt <- 0.0              # initial value for the exp term of pr(Y|A0, A+)
    pa0ap <- matrix(0, m, 1)  # Vector of log priors.

    for (i in 1:m)
    {
      # Grab the parameters in each structural equation.
      a0k <- varobj$A0.mode[,i]
      apk <- varobj$F.posterior[,i]

      # covariances of the free parameters in each equation
      S0bar <- varobj$H0inv.tilde[[i]]
      Spbar <- varobj$Hpinv.tilde[,,i]

      # Next 2 lines are the free parameters in each equation
      bk <- t(varobj$Ui[[i]])%*%a0k
      gbark <- varobj$Pi.tilde[[i]]%*%bk

      # Compute the exponential term.
      yexpt <- yexpt + 0.5*(t(a0k)%*%varobj$YY%*%a0k -
                            2*t(apk) %*% varobj$XY %*% a0k +
                            crossprod(apk, varobj$XX)%*%apk)
      # log prior

      pa0ap[i,] <- -0.5*(ncol(matrix(varobj$Ui[[i]], m)) +
                         nrow(varobj$F.posterior))*log(2*pi) +
                             0.5*log(abs(sum(diag(S0bar)))) +
                                 0.5*log(abs(sum(diag(Spbar)))) -
                                     0.5*(t(bk)%*%S0bar%*%bk + t(gk[,i] - gbark)%*%Spbar%*%(gk[,i]-gbark))
  }


  # Find log p(Y | A0, A+) --- this is the log likelihood at
  # the restricted parameters, so it is conditional on the model.  We
  # can use this later for computing the MDD and log posterior for the
  # models.

    U <- qr.R(qr(t(varobj$A0.mode)))
    ada <- sum(log(abs(diag(U))))

    log.llf <- -0.5*m*varobj$df*log(2*pi) + varobj$df*ada + yexpt

##     for (i in 1:varobj$df)
##     {
##         log.llf.restricted[i,] <- -(0.5*m)*log(2*pi) + ada -
##             0.5*crossprod(varobj$structural.innovations[i,])
##     }
##     log.marginal.llf.restricted <- cumsum(log.marginal.llf.restricted)

    # Now find Pr(A+ | Y, A0)
    totparam <- length(varobj$F.posterior)
    ld.covar.param <- sum(sapply(1:m, function(i) {
      determinant(varobj$Hpinv.posterior[[i]])$modulus }))

    bk <- a2b(varobj$A0.mode, varobj$Ui)
    Apexpt <- 0
    n0cum <- c(0,cumsum(varobj$n0))

    for (i in 1:m)
    {
        bj <- bk[(n0cum[i]+1):(n0cum[(i+1)])]
        tmp <- varobj$F.posterior[,i] - varobj$P.posterior[[i]]%*%bj
        Apexpt <- Apexpt - 0.5%*%t(tmp)%*%varobj$Hpinv.posterior[[i]]%*%tmp
    }

    pa.ya0 <- -0.5*totparam*log(2*pi) + ld.covar.param + Apexpt

    ################################################################################
    # Now find the pdf for each equation: p(a0_k |Y, a0_{not k})
    ################################################################################
    # Set up the gibbs sampler
    setup <- gibbs.setup.bsvar(varobj)
    Tinv <- setup$Tinv
    UT <- setup$UT

    logpa0.yao <- matrix(0, N2, m)  # storage for log Pr(A0_k \| Y A0_)
    vlog <- matrix(0, N2, m)       # storage across draws.
    A0gbs <- varobj$A0.mode
    Wout <- vector("list", m)
    bk <- a2b(varobj$A0.mode, varobj$Ui)
    b.free <- vector("list", m)
    for(i in 1:m)
    {
        b.free[[i]] <- bk[(n0cum[i]+1):(n0cum[(i+1)])]
    }

    # Build up the constant terms we need for each of the A0{k) calculations
    lgamma.term <- lgamma(0.5*(varobj$df+1))
    ldf <- log(varobj$df)
    df1 <- 0.5*(varobj$df+1)*log(2)
    constant1 <- 0.5*(varobj$df+varobj$n0)*ldf
    constant2 <- 0.5*(varobj$n0-1)*log(2*pi)
    constantterm <- constant1 - constant2 - df1 - lgamma.term

    logN2 <- log(N2)

      for(j in 1:N2)
      {
          Wout <- W.get(A0.posterior.obj$W.posterior, j)

          for (i in 1:(m-1))
          {
             # Loop over the m columns

              Vk <- solve(Tinv[[i]], Wout[[i]])
              gbeta <- solve(Vk, b.free[[i]])
              Vtr <- qr.R(qr(t(Vk)))

              vlog[j,i] <-
                  (constantterm[i] - log(abs(det(Vtr))) +
                   varobj$df*log(abs(gbeta[1])) -
                   0.5*varobj$df*crossprod(t(b.free[[i]]%*%solve(Vtr))))

          }


    # For the last or m^th column

          drawi <- drawA0(A0gbs, UT, varobj$df, varobj$n0, Wout)
          A0gbs <- drawi$A0gbs
          Wout <- drawi$W
          Vk <- solve(Tinv[[m]], Wout[[m]])
          gbeta <- solve(Vk, b.free[[m]])
          Vtr <- qr.R(qr(t(Vk)))

          vlog[j,m] <-
              (constantterm[m] - log(abs(det(Vtr))) +
               varobj$df*log(abs(gbeta[1])) -
               0.5*varobj$df*crossprod(t(b.free[[m]]%*%solve(Vtr))))

##           if (j%%1000==0)
##           { cat("Iteration = ", j, "\n"); }

      }
##           # Compute the modified harmonic mean of the max.
##           vlogmax <- max(vlog)
##           qlog <- vlog-vlogmax
##           vlogxhat <- vlogmax - logN2 + log(sum(exp(qlog)))
##           logpa0.yao[i,1] <- vlogxhat

    # Now compute the output objects

    log.prior <- sum(pa0ap)
#    lPy <- log.prior + log.llf - sum(logpa0.yao) - pa.ya0

    output <- list(log.prior=log.prior,
                   log.llf=log.llf,
                   log.posterior.Aplus=pa.ya0,
#                   log.marginal.data.density=lPy,
                   log.marginal.A0k=vlog)
#    class(output) <- c("posterior.fit.BSVAR")
    return(output)

}
